import time
import threading
from datetime import datetime, timedelta

from modsa_lib.core.command import Command
from modsa_lib.consts.sobeit import DIALOG_TABLIST, EVENT_GAME_INIT
from modsa_lib.core.event import Event
from modsa_lib.utils.json_helper import JsonFile
from modsa_lib.core.sobeit_api import GuiAPI, PlayerAPI


class LocalStats:
    json_path = "mod_sa/data/"

    def __init__(self):
        super().__init__()
        self.custom_fields = {}
        self.__json_file_name = None
        self.__json_file = None
        self.__game_start_dt = datetime.now()
        self.__today_dt = datetime.now()
        self.__last_save_dt = datetime.now() + timedelta(seconds=15)
        self.__today_data = None

        self.mystats_cmd = Command('mystats', self.command_callback)

    def update_data(self, param: str, value):
        pass

    def command_callback(self, args, cmd):
        session_time = int((datetime.now() - self.__game_start_dt).total_seconds())
        GuiAPI.show_samp_dialog(
            dialog_type=DIALOG_TABLIST,
            caption="Статистика",
            text=f'Текущая сессия:\t{self.parse_time(session_time)}\n'
            f'За сегодня:\t{self.parse_time(self.__today_data.get("time", 0))}\n'
            f'За всё время:\t{self.parse_time(self.get_all_time())}',
        )

    def get_json_path(self) -> str:
        return f'{self.json_path}{self.__json_file_name}'

    def get_all_time(self):
        total_seconds = 0
        for data in self.__json_file.data.values():
            total_seconds += data.get('time', 0)
        return total_seconds

    @staticmethod
    def parse_time(_time):
        time_str = ''
        years, temp_time = divmod(_time, 525600)
        if years:
            time_str += f'{years} г. '
        days, temp_time = divmod(temp_time, 86400)
        if days:
            time_str += f'{days} дн. '
        hours, temp_time = divmod(temp_time, 3600)
        if hours:
            time_str += f'{hours} ч. '
        minutes, temp_time = divmod(temp_time, 60)
        if minutes:
            time_str += f'{minutes} мин. '
        time_str += f'{temp_time} сек.'

        return time_str

    def save_data(self):
        time_elapsed = (datetime.now() - self.__last_save_dt).total_seconds()
        today_time = self.__today_data.get('time', 0)
        self.__today_data['time'] = today_time + int(time_elapsed)
        self.__json_file.write(self.__json_file.data)

    def update_today_data(self):
        date = f'{self.__today_dt.day}.{self.__today_dt.month}.{self.__today_dt.year}'
        if date not in self.__json_file.data:
            self.__json_file.data[date] = {}

        self.__today_data = self.__json_file.data[date]

    def stats_thread(self):
        player_name = PlayerAPI.get_name()
        self.__json_file_name = f'stats_{player_name}.json'
        self.__json_file = JsonFile(self.get_json_path())
        self.update_today_data()

        while True:
            date_now = datetime.now()
            if self.__today_dt.date() != date_now.date():
                self.__today_dt = datetime.now()
                self.update_today_data()

            if date_now - self.__last_save_dt > timedelta(seconds=15):
                self.save_data()
                self.__last_save_dt = date_now

            time.sleep(1.0)


local_player_stats = LocalStats()


@Event.register(EVENT_GAME_INIT)
def on_game_init():
    t = threading.Thread(target=local_player_stats.stats_thread, daemon=True)
    t.start()
