import json
import os
from json import JSONDecodeError


class JsonFile:
    data = None

    def __init__(self, filepath: str, default_data=None, autoload=True):
        """
        Args:
            filepath - путь к файлу
            default_data - данные по умолчанию (используются при создании файла)
            autoload - загрузить файл после инициализации
            use_settings_path - добавить в начало filepath путь к общей папке файлов
                с настройками (mod_sa/data/py_settings)
        """
        self.filepath = filepath
        self.default_data = default_data.copy() if default_data else {}
        if autoload:
            self.load_data()

    def load_data(self):
        self.data = self.read_or_create()

    def read_or_create(self, update=True):
        default_content_copy = self.default_data.copy() if self.default_data else self.default_data
        if not os.path.exists(self.filepath):
            folder_path = os.path.dirname(self.filepath)
            if folder_path:
                os.makedirs(folder_path, exist_ok=True)
            self.write(self.default_data if self.default_data else {})
            return default_content_copy

        try:
            file = open(self.filepath, 'r', encoding='utf8')
            data = json.load(file)
            if update and isinstance(self.default_data, dict):
                for key, value in self.default_data.items():
                    if key not in data:
                        data[key] = value

        except (OSError, JSONDecodeError):
            if self.default_data:
                self.write(self.default_data)
            return default_content_copy

        return data

    def save(self):
        self.write(self.data)

    def write(self, content: dict):
        with open(self.filepath, 'w', encoding='utf8') as file:
            file.write(json.dumps(content, ensure_ascii=False, indent=4))
